import { RPMessage } from "../utils/type";
import BaseProto from "./BaseProto";

export default class RegionProto extends BaseProto {
    override handleMessage(msg: RPMessage): void {
        if (msg.uid === undefined) {
            return this.sendError("0010", "uid is required");
        }

        switch (msg.cmd) {
            case "list":
                this.list(msg.uid, msg.data);
                break;
            case "inspect":
                this.inspect(msg.uid, msg.data);
                break;
            case "add":
                this.add(msg.uid, msg.token, msg.data);
                break;
            case "remove":
                this.remove(msg.uid, msg.data);
                break;
            case "create":
                this.create(msg.uid, msg.data);
                break;
            default:
                return this.sendUnknownCommand();
        }
    }
    
    private add(uid: string, token: string | undefined, data: any) {
        if (token === undefined) {
            return this.sendError("0010", "token is required");
        }
        if (this.manager.addToRegion(uid, token, data.regionId)) {
            this.sendRes("add", "0000", undefined);
        } else {
            this.sendError("0011", "region not found");
        }
    }

    private remove(uid: string, data: any) {
        throw new Error("Method not implemented.");
    }

    private list(uid: string, data: any) {
        const usersData = [];
        for (const [regionId, UserGroup] of this.manager.getIterRegions()) {
            usersData.push({ id: regionId, name: UserGroup.getName(),
                nowUsers: UserGroup.getNowUsers(),
                maxUsers: UserGroup.getMaxUsers(),
            });
        }
        this.sendRes("list", "0000", { "_": usersData });
    }

    private inspect(uid: string, data: any) {
        if (data?.regionId === undefined) {
            return this.sendError("0010", "regionId is required");
        }

        const region = this.manager.getIterRegion(data.regionId);
        if (region === undefined) {
            return this.sendError("0011", "region not found");
        }
        const usersData = [];
        for (const [userId, User] of region) {
            usersData.push({ id: userId, name: User.getName(),
            });
        }
        console.debug("inspect: ", usersData);
        this.sendRes("inspect", "0000", { "_": usersData });
    }

    private create(uid: string, data: any) {
    }

    private sendRes(cmd: string, code: string, data: any) {
        this.send({ type:'region', cmd, code, data});
    }
}