import { RPMessage } from "../utils/type";
import BaseProto from "./BaseProto";

export default class SessionProto extends BaseProto {
    override handleMessage(msg: RPMessage): void {
        if (msg.uid === undefined) {
            return this.sendError("0010", "uid is required");
        }

        switch (msg.cmd) {
            case "create":
                this.create(msg.uid, msg.token, msg.data);
                break;
            case "ackCreate":
                this.ackCreate(msg.uid, msg.code, msg.data);
                break;
            case "exit":
                this.exit(msg.uid, msg.token, msg.data);
                break;
            case "sendAll":
                this.sendAll(msg.uid, msg.token, msg.data);
                break;
            case "res":
                this.res(msg.cmd, msg.code, msg.data);
                break;
            default:
                this.sendUnknownCommand();
        }
    }
    res(cmd: string, code: string | undefined, data: any) {
        throw new Error("Method not implemented.");
    }

    ackCreate(uid: string, code: string | undefined, data: any) {
        let session = this.manager.getSession(data.sessionId);
        if (code !== "0000") {
            this.manager.removeSession(data.sessionId);
            return session?.sendToAll(JSON.stringify({
                type: "session",
                cmd: "ackCreate",
                code: "0000",
                data: { res: false, sessionId: data.sessionId }
            }));
        }

        let res = this.manager.waitForSession(data.sessionId, uid);
        if (res === false) {
            this.manager.removeSession(data.sessionId);
            return session?.sendToAll(JSON.stringify({
                type: "session",
                cmd: "ackCreate",
                code: "0000",
                data: { res: false, sessionId: data.sessionId }
            }));
        } else if (res === undefined) {
            return;
        }
        session?.sendToAll(JSON.stringify({
            type: "session",
            cmd: "ackCreate",
            code: "0000",
            data: {
                res: true,
                sessionId: data.sessionId,
            }
        }));
    }

    create(uid: string, token: string | undefined, data: any) {
        const usersId = data._ as string[]
        if (usersId === undefined) {
            return this.sendError("0011", "usersId is required");
        }
        let sessionid = this.manager.generateId();
        for (let id of usersId) {
            console.debug("id: ", id);
            let user = this.manager.getUser(id)
            if (user === undefined) {
                return this.sendError("0011", "user not found");
            }
            user.send(JSON.stringify({
                type: "session",
                cmd: "ackCreate",
                code: "0000",
                data: {
                    sessionId: sessionid,
                    reqUserId: uid,
                    reqUserName: user.getName(),
                }
            }));
        }
        this.manager.getUser(uid)?.send(JSON.stringify({
            type: "session",
            cmd: "create",
            code: "0000",
        }));

        var group = this.manager.createSession(uid, usersId.length + 1, sessionid);
        this.manager.addSession(sessionid, uid);
        group.on('GroupUserRemove', () => {
            group.sendToAll(JSON.stringify({
                type: "session",
                cmd: "exit",
                code: "0000",
                data: { "_": "sessionDelete" }
            }));
            this.manager.removeSession(sessionid);
        });
    }

    private sendRes(cmd: string, code: string, data: any) {
        this.send({ type:'session', cmd, code, data});
    }

    sendAll(uid: string, token: string | undefined, data: any) {
        var session = this.manager.getSession(data.sessionId);
        if (session == null) {
            this.sendRes("sendAll", "0404", { "msg": "session not found" });
            return;
        }
        session.sendToAll(JSON.stringify({
            type: "session",
            cmd: "sendAll",
            code: "0000",
            data: {
                sessionId: data.sessionId,
                reqUserId: uid,
                reqUserName: this.manager.getUser(uid)?.getName(),
                msg: data.msg,
            }
        }));
    }
    exit(uid: string, token: string | undefined, data: any) {
        throw new Error("Method not implemented.");
    }
}