import { RPMessage } from "../utils/type";
import BaseProto from "./BaseProto";

export default class UserProto extends BaseProto {
    override handleMessage(msg: RPMessage) {
        if (msg.cmd === "init") {
            this.init(msg.data);
            return;
        }

        if (msg.uid === undefined) {
            return this.sendError("0010", "uid is required");
        }
        switch (msg.cmd) {
            case "login":
                this.login(msg.uid, msg.data);
                break;
            case "logout":
                this.logout(msg.uid, msg.data);
                break;
            case "rename":
                this.rename(msg.uid, msg.data);
                break;
            case "exit":
                this.exit(msg.uid, msg.token??"token", msg.data);
                break;
            default:
                return this.sendUnknownCommand();
        }
    }
    private exit(uid: string, token: string, data: any) {
        let user =  this.manager.getUser(uid);
        if (user === undefined) {
            return;
        }
        if (token != user.getToken()) {
            return;
        }
        this.manager.removeUser(uid);
    }

    private rename(uid: string, data: any) {
        let user =  this.manager.getUser(uid);
        if (user === undefined) {
            return;
        }
        user.setName(data._ ? data._: user.getName());
        this.sendRes('rename', '0000', {"_": data});
    }
    
    private sendRes(cmd: string, code: string, data: any) {
        this.send({ type:'user', cmd, code, data});
    }

    private login(uid: string, data: any) {
        throw new Error("Method not implemented.");
    }
    private logout(uid: string, data: any) {
        throw new Error("Method not implemented.");
    }
    private init(data: any) {
        const fingerPrint = data?.fingerprint || "Godot Game";
        const userName = data?.name || 'unknown name';
        const { user } = this.manager.generateUser(userName, fingerPrint,
            this.ws, this.id);
        console.debug(`init user : ${this.id} ${userName}`);
        this.sendRes('init', '0000', {
            userId: this.id,
            userName,
            token: user.getToken(),
        });
    }
}