import { randomBytes, randomUUID } from "crypto";
import jwt from "jsonwebtoken";

export default class SecurityManager {
    private secretKey: string; // 用于JWT签名的密钥，请确保在生产环境中妥善保管

    constructor(secret?: string) {
        this.secretKey = secret ?? SecurityManager.generateSecretKey();
    }

    // 生成ID
    static generateId(): string {
        return randomUUID();
    }

    static generateSecretKey(byteLen: number = 32): string {
        return randomBytes(byteLen).toString('hex');
    }

    // 生成Token
    generateToken(userId: string, fingerprint: string, additionalInfo?: any): string {
        const payload = {
            userId,
            fingerprint,
            ...additionalInfo,
            iat: Math.floor(Date.now() / 1000), // 发行时间
            exp: Math.floor(Date.now() / 1000) + (60 * 60), // 过期时间，例如1小时后
        };
        return jwt.sign(payload, this.secretKey);
    }

    // 验证Token
    verifyToken(token: string): any | null {
        try {
            const decoded = jwt.verify(token, this.secretKey);
            return decoded;
        } catch (error) {
            console.error("Token verification failed:", error);
            return null;
        }
    }
}